/**
 * This file is the wrapper for the Java-calls to
 * the encryption primitives. 
 * @author Christian Grothoff
 * @author Krista Bennett
 * @file gproxy/CFuncts.c
 **/

#include "CFuncts.h"
#include "gnettypes.h"
#include "sessionkey.h"
#include <string.h>
#include <zlib.h>

/*
 * Class:     gproxy_CFuncts
 * Method:    crc32
 * Signature: ([B)I
 */
JNIEXPORT jint JNICALL Java_gproxy_CFuncts_crcNative
(JNIEnv * env, jclass jc, jbyteArray data) {
  uLong crc;
  jint result;
  char * buf;
  jboolean isCopy;
  jsize len;

  buf = (*env)->GetByteArrayElements(env, data, &isCopy);
  len = (*env)->GetArrayLength(env, data);

  crc = crc32(0L, Z_NULL,0);
  result = (jint) crc32(crc, (Bytef *)buf, len);
  (*env)->ReleaseByteArrayElements(env, data, buf, JNI_ABORT);
  /*  printf("crc: %d - %d\n",result,len);*/

  return result; 
}


/*
 * Class:     gproxy_CFuncts
 * Method:    hash
 * Signature: ([B)[B
 */
JNIEXPORT jbyteArray JNICALL Java_gproxy_CFuncts_hashNative
  (JNIEnv *env, jclass jc, jbyteArray str) {

  char * utf_string;
  char * hash_string;
  jboolean isCopy;
  jbyteArray ret_string;
  jsize old_strlen;

  utf_string = (*env)->GetByteArrayElements(env, str, &isCopy);
  old_strlen = (*env)->GetArrayLength(env, str);
  ret_string = (*env)->NewByteArray(env, 20);
  hash_string = (*env)->GetByteArrayElements(env, ret_string, &isCopy);
  
  RIPEMD160(utf_string, old_strlen, hash_string);
  (*env)->ReleaseByteArrayElements(env, str, utf_string, JNI_ABORT);
  (*env)->ReleaseByteArrayElements(env, ret_string, hash_string, JNI_COMMIT);
  
  return ret_string;
}

char iv[BLOWFISH_BLOCK_LENGTH];

/*
 * Class:     gproxy_CFuncts
 * Method:    decrypt
 * Signature: ([B[B)[B
 */
JNIEXPORT jbyteArray JNICALL Java_gproxy_CFuncts_decryptNative
(JNIEnv * env, jclass jc, 
 jbyteArray key, jbyteArray data) {
  unsigned char * utf_string_key;
  unsigned char * utf_string_data;
  unsigned char * decr_string;
  jboolean isCopy;
  jbyteArray target;

  EVP_CIPHER_CTX ctx;
  int outlen = 0;
  int tmplen;

  if ((key == 0) ||
      (data == 0))
    return NULL;
  utf_string_key = (*env)->GetByteArrayElements(env, key, &isCopy);
  utf_string_data = (*env)->GetByteArrayElements(env, data, &isCopy);
  /*
  for (tmplen=0;tmplen < 1024;tmplen++) 
    printf("%3ud-%d ",utf_string_data[tmplen],tmplen);
  printf("\n");
  */
  target = (*env)->NewByteArray(env, CONTENT_SIZE);
  decr_string = (*env)->GetByteArrayElements(env, target, &isCopy);

  /* get key and init value from the hash code */
  memcpy(iv, &utf_string_key[BF_KEYSIZE], BLOWFISH_BLOCK_LENGTH/2);
  memcpy(&iv[BLOWFISH_BLOCK_LENGTH/2], &utf_string_key[BF_KEYSIZE], 
         BLOWFISH_BLOCK_LENGTH/2);
  /* decrypt with the key */
  EVP_DecryptInit(&ctx, EVP_bf_cfb(), utf_string_key, iv);
  if (!EVP_DecryptUpdate(&ctx, decr_string,
                         &outlen, utf_string_data, CONTENT_SIZE)) {
    fprintf(stderr,
            "CFuncts.c-decryptBlock: EVP_DecryptUpdate failed!\n");
    (*env)->ReleaseByteArrayElements(env, key, utf_string_key, JNI_ABORT);
    (*env)->ReleaseByteArrayElements(env, data, utf_string_data, JNI_ABORT);
    (*env)->ReleaseByteArrayElements(env, target, decr_string, JNI_ABORT);
    return NULL;
  }
  
  if (!EVP_EncryptFinal(&ctx, decr_string + outlen, &tmplen)) {
    (*env)->ReleaseByteArrayElements(env, key, utf_string_key, JNI_ABORT);
    (*env)->ReleaseByteArrayElements(env, data, utf_string_data, JNI_ABORT);
    (*env)->ReleaseByteArrayElements(env, target, decr_string, JNI_ABORT);
    fprintf(stderr,
            "encrypt.c-encryptBlock: EVP_EncryptFinal failed!\n");
    return NULL;
  }
  
  EVP_CIPHER_CTX_cleanup(&ctx);
  
  (*env)->ReleaseByteArrayElements(env, target, decr_string, JNI_COMMIT); 
  (*env)->ReleaseByteArrayElements(env, key, utf_string_key, JNI_ABORT);
  (*env)->ReleaseByteArrayElements(env, data, utf_string_data, JNI_ABORT);
  return target;
}
