/**
 * GProxy main method and code
 * @file gproxy/GProxy.java
 **/
package gproxy;

/**
 * This is the main class of GProxy. 
 * @author Krista Bennett
 * @author Christian Grothoff
 **/
public class GProxy { 

    /**
     * Textmode or GUI version? The MainFrame class sets this to
     * true if we are having the GUI.
     **/ 
    public static final boolean DEBUG = false;

    /** 
     * This is the default GNet port
     **/
    public static final char DEFAULT_GNET_PORT = (char)2086; 

    /** 
     * This is the default host we connect to
     **/
    public static final String DEFAULT_GNET_HOST = "localhost";
 
    /**
     * The command line arguments.
     **/
    public static CommandLine cm;

    /**
     * The main method. Currently downloads the files corresponding
     * to the keywords in the command line. 
     **/
    public static void main(String[] args) {
	BuildThread bt = initNode(args);	
	RequestManager rm = new RequestManager(bt);
	DownloadModel dlm = new DownloadModel(rm);
	new MainWindow(dlm, bt).show();
    }

    /**
     * Initializes the connection with the GNet node to prepare for download
     * @param arguments from command line - query terms, temp directories, etc
     * @return background download thread
     **/
    public static BuildThread initNode(String[] args) {
	cm = new CommandLine(args);
	if (cm.getOption("help") != null) {
	    System.out.println("Call with: gproxy [OPTIONS]"); 
	    System.out.println("-host=HOSTNAME - set host running the gnet node");
	    System.out.println("-port=PORT     - set the port of the node");
	    System.out.println("-help          - show this page");
	    System.out.println("-loopback=DIR  - assemble content directly from DIR (no networking)");
	    System.out.println("Defaults are: host="+DEFAULT_GNET_HOST+
			                      " port="+DEFAULT_GNET_PORT);
	}
	String dir = cm.getOption("loopback");
	if (dir != null)
	    return new LoopbackBuildThread(dir);	
	BuildThread bt=null;
	try {
	    bt = new BuildThread(getGNetPort(),
				 getGNetHost());
	} catch (Exception e){
	    System.err.println("ERROR: "+e);
	    System.exit(-1);
	}
        bt.start(); // start background download thread, connection with GNet-node
	return bt;	
    }

    /**
     * This method returns the port to use for the connection to the GNet node.
     * @return port number to be used
     **/
    private static char getGNetPort() {       
         if (cm.getOption("port") == null)
              return DEFAULT_GNET_PORT;
         return (char)(new Integer(cm.getOption("port")).intValue());          
    }

    /**
     * This method returns the host to use for the connection to the GNet node. 
     * @return host to be used
     **/
    private static String getGNetHost() {       
	if (cm.getOption("host") == null)
	    return DEFAULT_GNET_HOST;
	return cm.getOption("host");
    }


} // End of GProxy

