/**
 * NodeInterface, the interface that all Nodes
 * (RNode, INode, Node) implement.
 * @file gproxy/NodeInterface.java
 **/
package gproxy;

/**
 * NodeInterface, the interface that all Nodes
 * (RNode, INode, Node) implement.
 * @author Krista Bennett
 * @author Christian Grothoff
 **/
public interface NodeInterface
    extends Listener {

    /**
     * Send out request for this node.
     **/
    public void request();

    /**
     * This code is called both to notify this node 
     * that one of its child-nodes received data.
     **/
    public void notifyReceivedData();

    /**
     * How may bytes are missing for this node to
     * be complete?
     **/
    public long missing();

    /**
     * How may bytes were received for this node so far?
     **/
    public long received();

    /**
     * Abort the download.
     **/
    public void abort();

} // end of NodeInterface
    
