/**
 * @file gproxy/QueryFrame.java
 **/
package gproxy;

import javax.swing.JScrollPane;

/**
 * This is the class representing the Download window of the
 * GProxy GUI. In this window, the user selects the files
 * that should be downloaded. There is one of these objects
 * per search-query of the user.
 * @author Christian Grothoff
 **/
public class QueryFrame 
    extends javax.swing.JFrame {

    /**
     * The searchList that this window displays.
     **/
    private final javax.swing.JList searchList;

    /**
     * The queryCoordinator that must be stopped
     * once this window is closed.
     **/
    private final QueryCoordinator qc_;

    /**
     * QueryModel for the List.
     **/
    private final QueryModel queryModel_;

    /** 
     * Create a new Download Window.
     * @param searchString the querystring for the window-title
     * @param queryModel_ the queryModel for the filelist
     **/
    public QueryFrame(QueryModel queryModel,
		      String searchString,
		      QueryCoordinator qc) {
	this.queryModel_ = queryModel;
	this.qc_ = qc;
	// init JFrame
        getContentPane().setLayout(new java.awt.BorderLayout());        
        setTitle("GProxy: "+searchString);        
	addWindowListener(new java.awt.event.WindowAdapter() {
		public void windowClosing
		    (java.awt.event.WindowEvent evt) {
		    qc_.terminate();		    
		}
	    }); 

	// init components
	javax.swing.JButton downloadButton 
	    = new javax.swing.JButton();
	searchList = new javax.swing.JList(queryModel_);
	downloadButton.setToolTipText("click here to download selected file");
        downloadButton.setText("download");
        downloadButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
		// go to search-list, find matching RootNode
		int[] selected = searchList.getSelectedIndices();
		queryModel_.download(selected);
            }
        });        
        getContentPane().add(downloadButton, java.awt.BorderLayout.NORTH);
        
	searchList.setForeground(java.awt.Color.black);
        searchList.setBackground(java.awt.Color.gray);
        searchList.setPreferredSize(new java.awt.Dimension(300, 500));
        searchList.setFixedCellWidth(100);
        searchList.setFixedCellHeight(14);
	JScrollPane scrollPane = new JScrollPane(searchList);
	getContentPane().add(scrollPane, java.awt.BorderLayout.CENTER);
	pack();
	setSize(600,400);
	show();
    }


} // end of QueryFrame
