/**
 * This Model represents the search results matching the given
 * query. It displays the results and reacts to choices by
 * the user by starting the download.
 * @file gproxy/QueryModel.java
 **/
package gproxy;

import java.util.Vector;

/**
 * This Model represents the search results matching the given
 * query. It displays the results and reacts to choices by
 * the user by starting the download.
 * @author Christian Grothoff
 **/
public class QueryModel 
    extends javax.swing.AbstractListModel {

    /**
     * The DownloadModel to notify about new downloads.
     **/
    private final DownloadModel dlm_;

    /**
     * Vector of all matching Root-Nodes to the search-query
     * that are currently on display.
     **/
    private final Vector roots_;

    /**
     * Create a new QueryModel.
     * @param dlm the DownloadModel to nofify once a download is started
     **/
    public QueryModel(DownloadModel dlm) {
	super();
	this.dlm_ = dlm;
	this.roots_ = new Vector();
    }

    /**
     * Return the n-th RNode to display for this QM.
     **/
    public Object getElementAt(int n) {
	return roots_.elementAt(n);
    }

    /**
     * Return the current number of matching search results.
     **/
    public int getSize() {
	return roots_.size();
    }

    /**
     * Add the given node to the list of displayed
     * results.
     * @param rn the RootNode of the result
     **/
    public synchronized void display(RootNode rn) {
	roots_.add(rn);
	fireIntervalAdded(this,
			  roots_.indexOf(rn),
			  roots_.indexOf(rn));
 	if (GProxy.DEBUG)
	    System.out.println("Displaying download: " + rn.toString());
   }

    /**
     * Download the selected entries (move them from
     * the querymodel to the download model).
     **/
    public synchronized void download(int[] selected) {
	for (int i=0;i<selected.length;i++) {
	    // locate root-node
	    RootNode rn = (RootNode) roots_.elementAt(selected[i]);
	    // remove from query list,
	    roots_.remove(rn);
	    // update view
	    fireIntervalRemoved(this,selected[i],selected[i]);
	    for (int j=0;j<selected.length;j++) 
		if (selected[j]>i)
		    selected[j]--;	    
	    // add to download list
	    dlm_.download(rn);
	}
    }

} // end of QueryModel
