/**
 * Main function to insert mp3 files into gnet.
 * @author Christian Grothoff
 * @file gproxy/insertmp3.c 
 **/

#include "insertfile.h"
#include "hostkey.h"
#include "lookup.h"
#include "cron.h"
#include "mp3tech.h"
#include <unistd.h>
#include <getopt.h>


/**
 * Prints the usage information for this command if the user errs.
 * Aborts the program.
 **/
void printhelp() {
  printf("USAGE: mp32gnet [OPTIONS] FILENAMES*\n");
  printf("-h, --help                    : print this page\n");
  printf("-c FILENAME, --config=FILENAME: load config file (defaults: ~/.gnet)\n");
  printf("-v, --version                 : print the version number\n");
  printf("-n, --noindex                 : do not index, full insertion\n");
}

/**
 * the main function to insert files into GNet.
 * @param argc number of arguments from the command line
 * @param argv command line arguments
 * @return return value from insertFile: 0: ok, -1: file not found 
 **/   
int main(int argc, char ** argv) {
  int res = 0;
  char* confFile;
  char* filename;
  char description[sizeof(id3tag)+100];
  char * keywords[4];
  int num_keywords;
  int c;
  int i;
  int j;
  mp3info info;
  
  unsigned int encode_content;
  
  /* set default config file */
  confFile = "~/.gnet";
  /* assume we encode content unless otherwise specified */
  encode_content = 1; /* index only (default) */
  filename = NULL;  
  while (1) {
    int option_index=0;
    static struct option long_options[] = {
      { "config",  1, 0, 'c' },
      { "version", 0, 0, 'v' },
      { "help",    0, 0, 'h' },
      { "noindex", 0, 0, 'n' },
      { 0,0,0,0 }
    };    
    c = getopt_long(argc,
		    argv, 
		    "vhnc:", 
		    long_options, 
		    &option_index);    
    if (c == -1) 
      break;  // No more flags to process
    switch(c) {
    case 'c': 
      confFile = optarg; 
      print("Configfile specified: %s.\n",confFile);
      break;
    case 'f': 
      filename = optarg; 
      break;
    case 'v': 
      print("Version v0.3.0 (beta), mp32gnet v0.0.1\n");
      return 0;
    case 'h': 
      printhelp(); 
      return 0;
    case 'n':
      encode_content = 0; /* NOT on demand */
      break;
    default: 
      print("Unknown option %c. Aborting.\nUse --help to get a list of options.\n",
	    c);
      return -1;
    } /* end of parsing commandline */
  } /* while (1) */
  if (argc - optind < 1) {
    printf("Not enough arguments.\n");
    printhelp();
    return -1;
  }
  readConfig(confFile);
  initRAND(); 
  if (encode_content == 1) {
    initCron();
    initLookup();
  }
  for (i=optind;i<argc;i++) {    
    filename = argv[i];
    printf("Working on file %s... ",filename);
    memset(&info,0,sizeof(mp3info));
    info.filename = filename;
    info.file = fopen(info.filename,"r");
    if (0 != get_mp3_info(&info)) {
      print("Error decoding mp3 header of %s, ignoring.\n",
	    info.filename);
      fclose(info.file);
      continue;
    }
    fclose(info.file);
    description[0] = '\0';
    strcat(description, info.id3.title);
    strcat(description, " - ");
    strcat(description, info.id3.artist);
    strcat(description, " - ");
    strcat(description, info.id3.album);
    strcat(description, ", ");
    strcat(description, info.id3.year);
    j = 0;
    if (strlen(info.id3.title) > 0)
      keywords[j++] = info.id3.title;
    if (strlen(info.id3.title) > 0)
       keywords[j++] = info.id3.artist;
     if (strlen(info.id3.title) > 0)
       keywords[j++] = info.id3.album;
     if (strlen(info.id3.title) > 0)
       keywords[j++] = info.id3.comment;
     if (j == 0) {
       print("No keywords in file, skipping.\n",
	     filename);
       continue;
     }
     num_keywords = j;
     while (j<4)
       keywords[j++] = "-";
     if (encode_content == 1) 
       encode_content = appendFilename(filename);
     res = insertFile(filename,
		      description,
		      keywords,
		      num_keywords,
		      encode_content);
     
     if (res == 0) {
       if (encode_content == 1)
	 printf("File successfully indexed with keywords %s, %s, %s and %s.\n", 
		keywords[0], keywords[1], keywords[2], keywords[3]);
       else
	 printf("File successfully inserted with keywords %s, %s, %s and %s.\n", 
		keywords[0], keywords[1], keywords[2], keywords[3]);
     } else
       printf("Error inserting the file (%d).\n",res);
  }
  return res;
}  

/* end of insertmain.c */
