/*
    mp3tech.c - Functions for handling MP3 files and most MP3 data
                structure manipulation.

    Copyright (C) 2000-2001  Cedric Tefft <cedric@earthling.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  ***************************************************************************

  This file is based in part on:

	* MP3Info 0.5 by Ricardo Cerqueira <rmc@rccn.net>
	* MP3Stat 0.9 by Ed Sweetman <safemode@voicenet.com> and 
			 Johannes Overmann <overmann@iname.com>

*/

#include "mp3tech.h"

/* Remove trailing whitespace from the end of a string */

char *unpad(char *string) {
  char *pos=string+strlen(string)-1;
  while(' ' == pos[0]) (pos--)[0]=0;
  return string;
}

int get_id3(mp3info *mp3) {
  int retcode=0;
  char fbuf[4];
  
  if(mp3->datasize >= 128) {
    if(fseek(mp3->file, -128, SEEK_END )) {
      fprintf(stderr,"ERROR: Couldn't read last 128 bytes of %s!!\n",mp3->filename);
      retcode |= 4;
    } else {
      fread(fbuf,1,3,mp3->file); fbuf[3] = '\0';
      mp3->id3.genre[0]=255;
      
      
      if (!strcmp((const char *)"TAG",(const char *)fbuf)) {
	
	
	mp3->id3_isvalid=1;
	mp3->datasize -= 128;
	fseek(mp3->file, -125, SEEK_END);
	fread(mp3->id3.title,1,30,mp3->file); mp3->id3.title[30] = '\0';
	fread(mp3->id3.artist,1,30,mp3->file); mp3->id3.artist[30] = '\0';
	fread(mp3->id3.album,1,30,mp3->file); mp3->id3.album[30] = '\0';
	fread(mp3->id3.year,1,4,mp3->file); mp3->id3.year[4] = '\0';
	fread(mp3->id3.comment,1,30,mp3->file); mp3->id3.comment[30] = '\0';
	if(mp3->id3.comment[28] == '\0') {
	  mp3->id3.track[0] = mp3->id3.comment[29];
	}
	fread(mp3->id3.genre,1,1,mp3->file);
	unpad(mp3->id3.title);
	unpad(mp3->id3.artist);
	unpad(mp3->id3.album);
	unpad(mp3->id3.year);
	unpad(mp3->id3.comment);
      }
    }
  }
  return retcode;
}


int get_mp3_info(mp3info *mp3) {
  struct stat filestat;

  stat(mp3->filename,&filestat);
  mp3->datasize=filestat.st_size;
  return get_id3(mp3);
}
