/**
 * This module is responsible to lookup queries.
 * @file gnet/primitive/lookup.c
 * @author Christian Grothoff
 **/

#ifndef LOOKUP_H
#define LOOKUP_H

#include "gnettypes.h"
#include "query.h"
#include "lookup_collision.h"
#include "cron.h"

/**
 * Initialize the lookup module.
 **/
void initLookup();

/**
 * Is space available on the drive? Returns the number of
 * blocks (1k) available. If there would be no space except
 * if less important content is discarded, this method 
 * *deletes* 1k of the less important content and returns 
 * 1. If there is only more important content, the
 * method returns 0.
 * @param importance how important is the content?
 **/
int spaceAvailable(int importance);

/**
 * Lookup the file on the local machine. If found,
 * send it back.
 * @param query the triple-hash of the content
 * @param replyTo the host that was asking for it
 * @param qp the polciy (for us: priority)
 * @returns 1 if found, 0 if not.
 **/
int tripleHashLookup(HashCode160 * query,
		     HostIdentity * replyTo,
		     QUERY_POLICY qp);


/**
 * Add an entry to the cache. The ->left and ->right
 * references are 0 when this method is called (new entry!)
 **/
void addEntry(ContentIndex * entry);

/**
 * Add a name to the list of filenames.
 **/
int appendFilename(char * filename);

#endif
/* end of lookup.h */
