/**
 * This module is responsible to handle queries.
 * @file gnet/include/query.h
 * @author Christian Grothoff
 **/

#ifndef QUERY_H
#define QUERY_H

#include "gnettypes.h"
#include "policy.h"
#include "storage.h"
#include "handler.h"
#include "lookup.h"

void initQuery();
 
/**
 * Check if the file the query is for is on the
 * local drive. If yes, send the reply and return 1,
 * otherwise return 0.
 **/
int localLookup(HashCode160 * query,
		HostIdentity * replyTo);
  
/**
 * Send a reply.
 * @param replyTo to whom to send the reply
 * @param content the data to send (1k)
 **/
void sendReply(HostIdentity * replyTo,
	       CONTENT_Block * content,
	       HashCode160 * hash);

/**
 * compare two hashcodes.
 **/
int equalsHashCode160(HashCode160 * a, 
		      HashCode160 * b);
 

/**
 * Execute the query. This method does NOT check the
 * QUERY Policy, it is used either after the policy
 * was checked or if the request is trusted (read:
 * comes from our GProxy).
 * @param qp the query policy (that defines what we'll do with the query)
 * @param msg the part that contains the query
 **/
void execQuery(QUERY_POLICY qp, QUERY_Message * msg);

/**
 * Execute the triple-hash query. This method does NOT check the
 * QUERY Policy, it is used either after the policy
 * was checked or if the request is trusted (read:
 * comes from our GProxy).
 * @param qp the query policy (that defines what we'll do with the query)
 * @param msg the part that contains the query
 **/
void exec3Query(QUERY_POLICY qp, QUERY_Message * msg);

#endif

/* end of query.h */
