/**
 * Routing interface. This is the interface that does the routing.
 * @author Christian Grothoff
 * @file include/routing.h
 **/

#ifndef ROUTING_H
#define ROUTING_H

#include "gnettypes.h"
#include "semaphore.h"
#include "query.h"
#include "cron.h"

/**
 * Initialize routing module.
 **/
void initRouting();

/**
 * Print the indirection table.
 **/
void printIndirectionTable(char * sign);

/**
 * Find out, if this query is already pending, if yes,
 * add this new return-to host to the reply indirection list
 * and return 1, otherwise return 0.
 * @param msg the Query to look for
 * @returns 1 if the query was already send out, 0 if not
 **/
int isPending(QUERY_Message *msg);

/**
 * We're going to forward an external request with
 * indirection. Thus this
 * node must keep track of the request for a while to
 * route results. This method should not call 
 * forwardQuery, the caller will.
 * @param msg the query we're going to forward
 **/
void indirectingQuery(QUERY_Message * msg);

/**
 * Content has arrived. We must decide if we want to
 * a) forward it to our GProxy
 * b) indirect it to other nodes
 * The routing module should know what to do.
 * @returns how good this content was (priority of the request)
 **/
int useContent(HostIdentity * hostId,
		CONTENT_Block * content,
		HashCode160 * hash);

 
/**
 * Compute the routingtable index of a query
 **/
unsigned int computeRoutingIndex(HashCode160 * hash);

#endif
/* end of routing.h */
