/**
 * This is the testcase for the outbuffer.
 * @author Christian Grothoff
 * @file gnet/database/test/identitytest.c
 **/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>

#include <pthread.h>

#include "connection.h"
#include "identity.h"
#include "keyservice.h"

#define TESTSTRING "Hello World\0"

/* #define DEBUG 1 */

int ok;

void testSendToHost() {
  HostIdentity me; 
  HELO_Message msg;

  createHELO(&msg);

  getHostIdentity(getPublicHostkey(),&me);
  bindAddress(&msg);
  sleep(1);
#ifdef DEBUG
  print("sending data..\n");
#endif
  sendToHost(&me, strlen(TESTSTRING)+1, TESTSTRING, 0, 0);
#ifdef DEBUG
  print("sleep, die\n");
#endif
  sleep(1);
  kill(ok,9);  
}

/*------------------------------------------------------------------------
 * passivesock - allocate & bind a server socket using GNET-UDP
 *------------------------------------------------------------------------
 */
int passivesock() {
  struct sockaddr_in sin;	/* an Internet endpoint address		*/
  int	s, type;	/* socket descriptor and socket type	*/
  

  memset(&sin, 0, sizeof(sin));
  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = INADDR_ANY;
  sin.sin_port = htons(getGnetPort());

  /* UDP */
  type = SOCK_DGRAM;

  /* Allocate a socket */
  s = socket(PF_INET, type, 17 /* UDP */);
  if (s < 0)
    errexit("can't create socket: %s\n", strerror(errno));
  
  /* Bind the socket */
  if (bind(s, (struct sockaddr *)&sin, sizeof(sin)) < 0)
    errexit("can't bind to port: %s\n",
	    strerror(errno));
  return s;
}

void startListener() {
  struct sockaddr_in incomming;
  int socket = passivesock();
  int addrlen = sizeof(incomming);  
  int size;
  char message[1500];

  if (0 != (ok=fork()))
    return; /* start 2nd process */
  bzero(&incomming,sizeof(struct sockaddr_in));
  if (socket < 0) {  
    print("unknown error reading from socket (%d)\n",
	  socket);
    exit(-1);    
  }
  size = recvfrom(socket, message, MESSAGE_SIZE, 0,
		  (struct sockaddr * )&incomming, &addrlen);
  close(socket);
  if ( (strcmp(&message[sizeof(MessagePack)],TESTSTRING)!=0)||
       (size != strlen(TESTSTRING)+1+sizeof(MessagePack)) ){
    print("Error: received %s (%d)\n", &message[sizeof(MessagePack)],
	  size);
    exit(-1);
  }
  print("testSendToHost ok\n");
  exit(0);
}

int main(int argc, char * argv[]) {
  int failureCount = 0;

  readConfig(DEFAULT_CONFIG_FILE);
  initAddress();
  initKeyService();
  initConnection();
  startListener();
  testSendToHost();

  if (failureCount == 0)
    return 0;
  else {
    print("\n\n%d TESTS FAILED!\n\n",failureCount);
    return -1;
  }


} // end of main
